from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.column_pii_contains_email_percent_check_spec import (
        ColumnPiiContainsEmailPercentCheckSpec,
    )
    from ..models.column_pii_contains_ip_4_percent_check_spec import (
        ColumnPiiContainsIp4PercentCheckSpec,
    )
    from ..models.column_pii_contains_ip_6_percent_check_spec import (
        ColumnPiiContainsIp6PercentCheckSpec,
    )
    from ..models.column_pii_contains_usa_phone_percent_check_spec import (
        ColumnPiiContainsUsaPhonePercentCheckSpec,
    )
    from ..models.column_pii_contains_usa_zipcode_percent_check_spec import (
        ColumnPiiContainsUsaZipcodePercentCheckSpec,
    )
    from ..models.column_pii_valid_email_percent_check_spec import (
        ColumnPiiValidEmailPercentCheckSpec,
    )
    from ..models.column_pii_valid_ip_4_address_percent_check_spec import (
        ColumnPiiValidIp4AddressPercentCheckSpec,
    )
    from ..models.column_pii_valid_ip_6_address_percent_check_spec import (
        ColumnPiiValidIp6AddressPercentCheckSpec,
    )
    from ..models.column_pii_valid_usa_phone_percent_check_spec import (
        ColumnPiiValidUsaPhonePercentCheckSpec,
    )
    from ..models.column_pii_valid_usa_zipcode_percent_check_spec import (
        ColumnPiiValidUsaZipcodePercentCheckSpec,
    )


T = TypeVar("T", bound="ColumnPiiProfilingChecksSpec")


@attr.s(auto_attribs=True)
class ColumnPiiProfilingChecksSpec:
    """
    Attributes:
        profile_valid_usa_phone_percent (Union[Unset, ColumnPiiValidUsaPhonePercentCheckSpec]):
        profile_contains_usa_phone_percent (Union[Unset, ColumnPiiContainsUsaPhonePercentCheckSpec]):
        profile_valid_usa_zipcode_percent (Union[Unset, ColumnPiiValidUsaZipcodePercentCheckSpec]):
        profile_contains_usa_zipcode_percent (Union[Unset, ColumnPiiContainsUsaZipcodePercentCheckSpec]):
        profile_valid_email_percent (Union[Unset, ColumnPiiValidEmailPercentCheckSpec]):
        profile_contains_email_percent (Union[Unset, ColumnPiiContainsEmailPercentCheckSpec]):
        profile_valid_ip4_address_percent (Union[Unset, ColumnPiiValidIp4AddressPercentCheckSpec]):
        profile_contains_ip4_percent (Union[Unset, ColumnPiiContainsIp4PercentCheckSpec]):
        profile_valid_ip6_address_percent (Union[Unset, ColumnPiiValidIp6AddressPercentCheckSpec]):
        profile_contains_ip6_percent (Union[Unset, ColumnPiiContainsIp6PercentCheckSpec]):
    """

    profile_valid_usa_phone_percent: Union[
        Unset, "ColumnPiiValidUsaPhonePercentCheckSpec"
    ] = UNSET
    profile_contains_usa_phone_percent: Union[
        Unset, "ColumnPiiContainsUsaPhonePercentCheckSpec"
    ] = UNSET
    profile_valid_usa_zipcode_percent: Union[
        Unset, "ColumnPiiValidUsaZipcodePercentCheckSpec"
    ] = UNSET
    profile_contains_usa_zipcode_percent: Union[
        Unset, "ColumnPiiContainsUsaZipcodePercentCheckSpec"
    ] = UNSET
    profile_valid_email_percent: Union[
        Unset, "ColumnPiiValidEmailPercentCheckSpec"
    ] = UNSET
    profile_contains_email_percent: Union[
        Unset, "ColumnPiiContainsEmailPercentCheckSpec"
    ] = UNSET
    profile_valid_ip4_address_percent: Union[
        Unset, "ColumnPiiValidIp4AddressPercentCheckSpec"
    ] = UNSET
    profile_contains_ip4_percent: Union[
        Unset, "ColumnPiiContainsIp4PercentCheckSpec"
    ] = UNSET
    profile_valid_ip6_address_percent: Union[
        Unset, "ColumnPiiValidIp6AddressPercentCheckSpec"
    ] = UNSET
    profile_contains_ip6_percent: Union[
        Unset, "ColumnPiiContainsIp6PercentCheckSpec"
    ] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        profile_valid_usa_phone_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_valid_usa_phone_percent, Unset):
            profile_valid_usa_phone_percent = (
                self.profile_valid_usa_phone_percent.to_dict()
            )

        profile_contains_usa_phone_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_contains_usa_phone_percent, Unset):
            profile_contains_usa_phone_percent = (
                self.profile_contains_usa_phone_percent.to_dict()
            )

        profile_valid_usa_zipcode_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_valid_usa_zipcode_percent, Unset):
            profile_valid_usa_zipcode_percent = (
                self.profile_valid_usa_zipcode_percent.to_dict()
            )

        profile_contains_usa_zipcode_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_contains_usa_zipcode_percent, Unset):
            profile_contains_usa_zipcode_percent = (
                self.profile_contains_usa_zipcode_percent.to_dict()
            )

        profile_valid_email_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_valid_email_percent, Unset):
            profile_valid_email_percent = self.profile_valid_email_percent.to_dict()

        profile_contains_email_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_contains_email_percent, Unset):
            profile_contains_email_percent = (
                self.profile_contains_email_percent.to_dict()
            )

        profile_valid_ip4_address_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_valid_ip4_address_percent, Unset):
            profile_valid_ip4_address_percent = (
                self.profile_valid_ip4_address_percent.to_dict()
            )

        profile_contains_ip4_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_contains_ip4_percent, Unset):
            profile_contains_ip4_percent = self.profile_contains_ip4_percent.to_dict()

        profile_valid_ip6_address_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_valid_ip6_address_percent, Unset):
            profile_valid_ip6_address_percent = (
                self.profile_valid_ip6_address_percent.to_dict()
            )

        profile_contains_ip6_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.profile_contains_ip6_percent, Unset):
            profile_contains_ip6_percent = self.profile_contains_ip6_percent.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if profile_valid_usa_phone_percent is not UNSET:
            field_dict[
                "profile_valid_usa_phone_percent"
            ] = profile_valid_usa_phone_percent
        if profile_contains_usa_phone_percent is not UNSET:
            field_dict[
                "profile_contains_usa_phone_percent"
            ] = profile_contains_usa_phone_percent
        if profile_valid_usa_zipcode_percent is not UNSET:
            field_dict[
                "profile_valid_usa_zipcode_percent"
            ] = profile_valid_usa_zipcode_percent
        if profile_contains_usa_zipcode_percent is not UNSET:
            field_dict[
                "profile_contains_usa_zipcode_percent"
            ] = profile_contains_usa_zipcode_percent
        if profile_valid_email_percent is not UNSET:
            field_dict["profile_valid_email_percent"] = profile_valid_email_percent
        if profile_contains_email_percent is not UNSET:
            field_dict[
                "profile_contains_email_percent"
            ] = profile_contains_email_percent
        if profile_valid_ip4_address_percent is not UNSET:
            field_dict[
                "profile_valid_ip4_address_percent"
            ] = profile_valid_ip4_address_percent
        if profile_contains_ip4_percent is not UNSET:
            field_dict["profile_contains_ip4_percent"] = profile_contains_ip4_percent
        if profile_valid_ip6_address_percent is not UNSET:
            field_dict[
                "profile_valid_ip6_address_percent"
            ] = profile_valid_ip6_address_percent
        if profile_contains_ip6_percent is not UNSET:
            field_dict["profile_contains_ip6_percent"] = profile_contains_ip6_percent

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_pii_contains_email_percent_check_spec import (
            ColumnPiiContainsEmailPercentCheckSpec,
        )
        from ..models.column_pii_contains_ip_4_percent_check_spec import (
            ColumnPiiContainsIp4PercentCheckSpec,
        )
        from ..models.column_pii_contains_ip_6_percent_check_spec import (
            ColumnPiiContainsIp6PercentCheckSpec,
        )
        from ..models.column_pii_contains_usa_phone_percent_check_spec import (
            ColumnPiiContainsUsaPhonePercentCheckSpec,
        )
        from ..models.column_pii_contains_usa_zipcode_percent_check_spec import (
            ColumnPiiContainsUsaZipcodePercentCheckSpec,
        )
        from ..models.column_pii_valid_email_percent_check_spec import (
            ColumnPiiValidEmailPercentCheckSpec,
        )
        from ..models.column_pii_valid_ip_4_address_percent_check_spec import (
            ColumnPiiValidIp4AddressPercentCheckSpec,
        )
        from ..models.column_pii_valid_ip_6_address_percent_check_spec import (
            ColumnPiiValidIp6AddressPercentCheckSpec,
        )
        from ..models.column_pii_valid_usa_phone_percent_check_spec import (
            ColumnPiiValidUsaPhonePercentCheckSpec,
        )
        from ..models.column_pii_valid_usa_zipcode_percent_check_spec import (
            ColumnPiiValidUsaZipcodePercentCheckSpec,
        )

        d = src_dict.copy()
        _profile_valid_usa_phone_percent = d.pop(
            "profile_valid_usa_phone_percent", UNSET
        )
        profile_valid_usa_phone_percent: Union[
            Unset, ColumnPiiValidUsaPhonePercentCheckSpec
        ]
        if isinstance(_profile_valid_usa_phone_percent, Unset):
            profile_valid_usa_phone_percent = UNSET
        else:
            profile_valid_usa_phone_percent = (
                ColumnPiiValidUsaPhonePercentCheckSpec.from_dict(
                    _profile_valid_usa_phone_percent
                )
            )

        _profile_contains_usa_phone_percent = d.pop(
            "profile_contains_usa_phone_percent", UNSET
        )
        profile_contains_usa_phone_percent: Union[
            Unset, ColumnPiiContainsUsaPhonePercentCheckSpec
        ]
        if isinstance(_profile_contains_usa_phone_percent, Unset):
            profile_contains_usa_phone_percent = UNSET
        else:
            profile_contains_usa_phone_percent = (
                ColumnPiiContainsUsaPhonePercentCheckSpec.from_dict(
                    _profile_contains_usa_phone_percent
                )
            )

        _profile_valid_usa_zipcode_percent = d.pop(
            "profile_valid_usa_zipcode_percent", UNSET
        )
        profile_valid_usa_zipcode_percent: Union[
            Unset, ColumnPiiValidUsaZipcodePercentCheckSpec
        ]
        if isinstance(_profile_valid_usa_zipcode_percent, Unset):
            profile_valid_usa_zipcode_percent = UNSET
        else:
            profile_valid_usa_zipcode_percent = (
                ColumnPiiValidUsaZipcodePercentCheckSpec.from_dict(
                    _profile_valid_usa_zipcode_percent
                )
            )

        _profile_contains_usa_zipcode_percent = d.pop(
            "profile_contains_usa_zipcode_percent", UNSET
        )
        profile_contains_usa_zipcode_percent: Union[
            Unset, ColumnPiiContainsUsaZipcodePercentCheckSpec
        ]
        if isinstance(_profile_contains_usa_zipcode_percent, Unset):
            profile_contains_usa_zipcode_percent = UNSET
        else:
            profile_contains_usa_zipcode_percent = (
                ColumnPiiContainsUsaZipcodePercentCheckSpec.from_dict(
                    _profile_contains_usa_zipcode_percent
                )
            )

        _profile_valid_email_percent = d.pop("profile_valid_email_percent", UNSET)
        profile_valid_email_percent: Union[Unset, ColumnPiiValidEmailPercentCheckSpec]
        if isinstance(_profile_valid_email_percent, Unset):
            profile_valid_email_percent = UNSET
        else:
            profile_valid_email_percent = ColumnPiiValidEmailPercentCheckSpec.from_dict(
                _profile_valid_email_percent
            )

        _profile_contains_email_percent = d.pop("profile_contains_email_percent", UNSET)
        profile_contains_email_percent: Union[
            Unset, ColumnPiiContainsEmailPercentCheckSpec
        ]
        if isinstance(_profile_contains_email_percent, Unset):
            profile_contains_email_percent = UNSET
        else:
            profile_contains_email_percent = (
                ColumnPiiContainsEmailPercentCheckSpec.from_dict(
                    _profile_contains_email_percent
                )
            )

        _profile_valid_ip4_address_percent = d.pop(
            "profile_valid_ip4_address_percent", UNSET
        )
        profile_valid_ip4_address_percent: Union[
            Unset, ColumnPiiValidIp4AddressPercentCheckSpec
        ]
        if isinstance(_profile_valid_ip4_address_percent, Unset):
            profile_valid_ip4_address_percent = UNSET
        else:
            profile_valid_ip4_address_percent = (
                ColumnPiiValidIp4AddressPercentCheckSpec.from_dict(
                    _profile_valid_ip4_address_percent
                )
            )

        _profile_contains_ip4_percent = d.pop("profile_contains_ip4_percent", UNSET)
        profile_contains_ip4_percent: Union[Unset, ColumnPiiContainsIp4PercentCheckSpec]
        if isinstance(_profile_contains_ip4_percent, Unset):
            profile_contains_ip4_percent = UNSET
        else:
            profile_contains_ip4_percent = (
                ColumnPiiContainsIp4PercentCheckSpec.from_dict(
                    _profile_contains_ip4_percent
                )
            )

        _profile_valid_ip6_address_percent = d.pop(
            "profile_valid_ip6_address_percent", UNSET
        )
        profile_valid_ip6_address_percent: Union[
            Unset, ColumnPiiValidIp6AddressPercentCheckSpec
        ]
        if isinstance(_profile_valid_ip6_address_percent, Unset):
            profile_valid_ip6_address_percent = UNSET
        else:
            profile_valid_ip6_address_percent = (
                ColumnPiiValidIp6AddressPercentCheckSpec.from_dict(
                    _profile_valid_ip6_address_percent
                )
            )

        _profile_contains_ip6_percent = d.pop("profile_contains_ip6_percent", UNSET)
        profile_contains_ip6_percent: Union[Unset, ColumnPiiContainsIp6PercentCheckSpec]
        if isinstance(_profile_contains_ip6_percent, Unset):
            profile_contains_ip6_percent = UNSET
        else:
            profile_contains_ip6_percent = (
                ColumnPiiContainsIp6PercentCheckSpec.from_dict(
                    _profile_contains_ip6_percent
                )
            )

        column_pii_profiling_checks_spec = cls(
            profile_valid_usa_phone_percent=profile_valid_usa_phone_percent,
            profile_contains_usa_phone_percent=profile_contains_usa_phone_percent,
            profile_valid_usa_zipcode_percent=profile_valid_usa_zipcode_percent,
            profile_contains_usa_zipcode_percent=profile_contains_usa_zipcode_percent,
            profile_valid_email_percent=profile_valid_email_percent,
            profile_contains_email_percent=profile_contains_email_percent,
            profile_valid_ip4_address_percent=profile_valid_ip4_address_percent,
            profile_contains_ip4_percent=profile_contains_ip4_percent,
            profile_valid_ip6_address_percent=profile_valid_ip6_address_percent,
            profile_contains_ip6_percent=profile_contains_ip6_percent,
        )

        column_pii_profiling_checks_spec.additional_properties = d
        return column_pii_profiling_checks_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.table_change_row_count_check_spec import TableChangeRowCountCheckSpec
    from ..models.table_row_count_check_spec import TableRowCountCheckSpec


T = TypeVar("T", bound="TableVolumeMonthlyRecurringChecksSpec")


@attr.s(auto_attribs=True)
class TableVolumeMonthlyRecurringChecksSpec:
    """
    Attributes:
        monthly_row_count (Union[Unset, TableRowCountCheckSpec]):
        monthly_row_count_change (Union[Unset, TableChangeRowCountCheckSpec]):
    """

    monthly_row_count: Union[Unset, "TableRowCountCheckSpec"] = UNSET
    monthly_row_count_change: Union[Unset, "TableChangeRowCountCheckSpec"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        monthly_row_count: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.monthly_row_count, Unset):
            monthly_row_count = self.monthly_row_count.to_dict()

        monthly_row_count_change: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.monthly_row_count_change, Unset):
            monthly_row_count_change = self.monthly_row_count_change.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if monthly_row_count is not UNSET:
            field_dict["monthly_row_count"] = monthly_row_count
        if monthly_row_count_change is not UNSET:
            field_dict["monthly_row_count_change"] = monthly_row_count_change

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.table_change_row_count_check_spec import (
            TableChangeRowCountCheckSpec,
        )
        from ..models.table_row_count_check_spec import TableRowCountCheckSpec

        d = src_dict.copy()
        _monthly_row_count = d.pop("monthly_row_count", UNSET)
        monthly_row_count: Union[Unset, TableRowCountCheckSpec]
        if isinstance(_monthly_row_count, Unset):
            monthly_row_count = UNSET
        else:
            monthly_row_count = TableRowCountCheckSpec.from_dict(_monthly_row_count)

        _monthly_row_count_change = d.pop("monthly_row_count_change", UNSET)
        monthly_row_count_change: Union[Unset, TableChangeRowCountCheckSpec]
        if isinstance(_monthly_row_count_change, Unset):
            monthly_row_count_change = UNSET
        else:
            monthly_row_count_change = TableChangeRowCountCheckSpec.from_dict(
                _monthly_row_count_change
            )

        table_volume_monthly_recurring_checks_spec = cls(
            monthly_row_count=monthly_row_count,
            monthly_row_count_change=monthly_row_count_change,
        )

        table_volume_monthly_recurring_checks_spec.additional_properties = d
        return table_volume_monthly_recurring_checks_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class EffectiveScheduleModelScheduleGroup(str, Enum):
    PARTITIONED_DAILY = "partitioned_daily"
    PARTITIONED_MONTHLY = "partitioned_monthly"
    PROFILING = "profiling"
    RECURRING_DAILY = "recurring_daily"
    RECURRING_MONTHLY = "recurring_monthly"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="ErrorDetailedSingleModel")


@attr.s(auto_attribs=True)
class ErrorDetailedSingleModel:
    """
    Attributes:
        actual_value (Union[Unset, float]): Actual value.
        expected_value (Union[Unset, float]): Expected value.
        column_name (Union[Unset, str]): Column name.
        data_group (Union[Unset, str]): Data group.
        duration_ms (Union[Unset, int]): Duration (ms).
        executed_at (Union[Unset, int]): Executed at.
        time_gradient (Union[Unset, str]): Time gradient.
        time_period (Union[Unset, datetime.datetime]): Time period.
        provider (Union[Unset, str]): Provider.
        quality_dimension (Union[Unset, str]): Quality dimension.
        sensor_name (Union[Unset, str]): Sensor name.
        readout_id (Union[Unset, str]): Sensor readout id.
        error_message (Union[Unset, str]): Error message.
        error_source (Union[Unset, str]): Error source.
        error_timestamp (Union[Unset, datetime.datetime]): Error timestamp.
    """

    actual_value: Union[Unset, float] = UNSET
    expected_value: Union[Unset, float] = UNSET
    column_name: Union[Unset, str] = UNSET
    data_group: Union[Unset, str] = UNSET
    duration_ms: Union[Unset, int] = UNSET
    executed_at: Union[Unset, int] = UNSET
    time_gradient: Union[Unset, str] = UNSET
    time_period: Union[Unset, datetime.datetime] = UNSET
    provider: Union[Unset, str] = UNSET
    quality_dimension: Union[Unset, str] = UNSET
    sensor_name: Union[Unset, str] = UNSET
    readout_id: Union[Unset, str] = UNSET
    error_message: Union[Unset, str] = UNSET
    error_source: Union[Unset, str] = UNSET
    error_timestamp: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        actual_value = self.actual_value
        expected_value = self.expected_value
        column_name = self.column_name
        data_group = self.data_group
        duration_ms = self.duration_ms
        executed_at = self.executed_at
        time_gradient = self.time_gradient
        time_period: Union[Unset, str] = UNSET
        if not isinstance(self.time_period, Unset):
            time_period = self.time_period.isoformat()

        provider = self.provider
        quality_dimension = self.quality_dimension
        sensor_name = self.sensor_name
        readout_id = self.readout_id
        error_message = self.error_message
        error_source = self.error_source
        error_timestamp: Union[Unset, str] = UNSET
        if not isinstance(self.error_timestamp, Unset):
            error_timestamp = self.error_timestamp.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if actual_value is not UNSET:
            field_dict["actualValue"] = actual_value
        if expected_value is not UNSET:
            field_dict["expectedValue"] = expected_value
        if column_name is not UNSET:
            field_dict["columnName"] = column_name
        if data_group is not UNSET:
            field_dict["dataGroup"] = data_group
        if duration_ms is not UNSET:
            field_dict["durationMs"] = duration_ms
        if executed_at is not UNSET:
            field_dict["executedAt"] = executed_at
        if time_gradient is not UNSET:
            field_dict["timeGradient"] = time_gradient
        if time_period is not UNSET:
            field_dict["timePeriod"] = time_period
        if provider is not UNSET:
            field_dict["provider"] = provider
        if quality_dimension is not UNSET:
            field_dict["qualityDimension"] = quality_dimension
        if sensor_name is not UNSET:
            field_dict["sensorName"] = sensor_name
        if readout_id is not UNSET:
            field_dict["readoutId"] = readout_id
        if error_message is not UNSET:
            field_dict["errorMessage"] = error_message
        if error_source is not UNSET:
            field_dict["errorSource"] = error_source
        if error_timestamp is not UNSET:
            field_dict["errorTimestamp"] = error_timestamp

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        actual_value = d.pop("actualValue", UNSET)

        expected_value = d.pop("expectedValue", UNSET)

        column_name = d.pop("columnName", UNSET)

        data_group = d.pop("dataGroup", UNSET)

        duration_ms = d.pop("durationMs", UNSET)

        executed_at = d.pop("executedAt", UNSET)

        time_gradient = d.pop("timeGradient", UNSET)

        _time_period = d.pop("timePeriod", UNSET)
        time_period: Union[Unset, datetime.datetime]
        if isinstance(_time_period, Unset):
            time_period = UNSET
        else:
            time_period = isoparse(_time_period)

        provider = d.pop("provider", UNSET)

        quality_dimension = d.pop("qualityDimension", UNSET)

        sensor_name = d.pop("sensorName", UNSET)

        readout_id = d.pop("readoutId", UNSET)

        error_message = d.pop("errorMessage", UNSET)

        error_source = d.pop("errorSource", UNSET)

        _error_timestamp = d.pop("errorTimestamp", UNSET)
        error_timestamp: Union[Unset, datetime.datetime]
        if isinstance(_error_timestamp, Unset):
            error_timestamp = UNSET
        else:
            error_timestamp = isoparse(_error_timestamp)

        error_detailed_single_model = cls(
            actual_value=actual_value,
            expected_value=expected_value,
            column_name=column_name,
            data_group=data_group,
            duration_ms=duration_ms,
            executed_at=executed_at,
            time_gradient=time_gradient,
            time_period=time_period,
            provider=provider,
            quality_dimension=quality_dimension,
            sensor_name=sensor_name,
            readout_id=readout_id,
            error_message=error_message,
            error_source=error_source,
            error_timestamp=error_timestamp,
        )

        error_detailed_single_model.additional_properties = d
        return error_detailed_single_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.rule_model_mode import RuleModelMode
from ..models.rule_model_type import RuleModelType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.parameter_definition_spec import ParameterDefinitionSpec
    from ..models.rule_model_parameters import RuleModelParameters
    from ..models.rule_time_window_settings_spec import RuleTimeWindowSettingsSpec


T = TypeVar("T", bound="RuleModel")


@attr.s(auto_attribs=True)
class RuleModel:
    """Rule model

    Attributes:
        rule_name (Union[Unset, str]): Rule name
        rule_python_module_content (Union[Unset, str]): Rule Python module content
        type (Union[Unset, RuleModelType]): Rule runner type
        java_class_name (Union[Unset, str]): Java class name for a rule runner that will execute the sensor. The "type"
            must be "java_class".
        mode (Union[Unset, RuleModelMode]): Rule historic (past) values mode. A rule may require just the current sensor
            readout or use sensor readouts from past periods to perform prediction. The number of time windows is configured
            in the time_window setting.
        time_window (Union[Unset, RuleTimeWindowSettingsSpec]):
        fields (Union[Unset, List['ParameterDefinitionSpec']]): List of fields that are parameters of a custom rule.
            Those fields are used by the DQO UI to display the data quality check editing screens with proper UI controls
            for all required fields.
        parameters (Union[Unset, RuleModelParameters]): Additional rule parameters
        custom (Union[Unset, bool]): This rule has a custom (user level) definition.
        built_in (Union[Unset, bool]): This rule has is a built-in rule.
    """

    rule_name: Union[Unset, str] = UNSET
    rule_python_module_content: Union[Unset, str] = UNSET
    type: Union[Unset, RuleModelType] = UNSET
    java_class_name: Union[Unset, str] = UNSET
    mode: Union[Unset, RuleModelMode] = UNSET
    time_window: Union[Unset, "RuleTimeWindowSettingsSpec"] = UNSET
    fields: Union[Unset, List["ParameterDefinitionSpec"]] = UNSET
    parameters: Union[Unset, "RuleModelParameters"] = UNSET
    custom: Union[Unset, bool] = UNSET
    built_in: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        rule_name = self.rule_name
        rule_python_module_content = self.rule_python_module_content
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        java_class_name = self.java_class_name
        mode: Union[Unset, str] = UNSET
        if not isinstance(self.mode, Unset):
            mode = self.mode.value

        time_window: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.time_window, Unset):
            time_window = self.time_window.to_dict()

        fields: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.fields, Unset):
            fields = []
            for fields_item_data in self.fields:
                fields_item = fields_item_data.to_dict()

                fields.append(fields_item)

        parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.parameters, Unset):
            parameters = self.parameters.to_dict()

        custom = self.custom
        built_in = self.built_in

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if rule_name is not UNSET:
            field_dict["rule_name"] = rule_name
        if rule_python_module_content is not UNSET:
            field_dict["rule_python_module_content"] = rule_python_module_content
        if type is not UNSET:
            field_dict["type"] = type
        if java_class_name is not UNSET:
            field_dict["java_class_name"] = java_class_name
        if mode is not UNSET:
            field_dict["mode"] = mode
        if time_window is not UNSET:
            field_dict["time_window"] = time_window
        if fields is not UNSET:
            field_dict["fields"] = fields
        if parameters is not UNSET:
            field_dict["parameters"] = parameters
        if custom is not UNSET:
            field_dict["custom"] = custom
        if built_in is not UNSET:
            field_dict["built_in"] = built_in

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.parameter_definition_spec import ParameterDefinitionSpec
        from ..models.rule_model_parameters import RuleModelParameters
        from ..models.rule_time_window_settings_spec import RuleTimeWindowSettingsSpec

        d = src_dict.copy()
        rule_name = d.pop("rule_name", UNSET)

        rule_python_module_content = d.pop("rule_python_module_content", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, RuleModelType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = RuleModelType(_type)

        java_class_name = d.pop("java_class_name", UNSET)

        _mode = d.pop("mode", UNSET)
        mode: Union[Unset, RuleModelMode]
        if isinstance(_mode, Unset):
            mode = UNSET
        else:
            mode = RuleModelMode(_mode)

        _time_window = d.pop("time_window", UNSET)
        time_window: Union[Unset, RuleTimeWindowSettingsSpec]
        if isinstance(_time_window, Unset):
            time_window = UNSET
        else:
            time_window = RuleTimeWindowSettingsSpec.from_dict(_time_window)

        fields = []
        _fields = d.pop("fields", UNSET)
        for fields_item_data in _fields or []:
            fields_item = ParameterDefinitionSpec.from_dict(fields_item_data)

            fields.append(fields_item)

        _parameters = d.pop("parameters", UNSET)
        parameters: Union[Unset, RuleModelParameters]
        if isinstance(_parameters, Unset):
            parameters = UNSET
        else:
            parameters = RuleModelParameters.from_dict(_parameters)

        custom = d.pop("custom", UNSET)

        built_in = d.pop("built_in", UNSET)

        rule_model = cls(
            rule_name=rule_name,
            rule_python_module_content=rule_python_module_content,
            type=type,
            java_class_name=java_class_name,
            mode=mode,
            time_window=time_window,
            fields=fields,
            parameters=parameters,
            custom=custom,
            built_in=built_in,
        )

        rule_model.additional_properties = d
        return rule_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.column_not_nulls_count_check_spec import ColumnNotNullsCountCheckSpec
    from ..models.column_not_nulls_percent_check_spec import (
        ColumnNotNullsPercentCheckSpec,
    )
    from ..models.column_nulls_count_check_spec import ColumnNullsCountCheckSpec
    from ..models.column_nulls_percent_check_spec import ColumnNullsPercentCheckSpec


T = TypeVar("T", bound="ColumnNullsMonthlyRecurringChecksSpec")


@attr.s(auto_attribs=True)
class ColumnNullsMonthlyRecurringChecksSpec:
    """
    Attributes:
        monthly_nulls_count (Union[Unset, ColumnNullsCountCheckSpec]):
        monthly_nulls_percent (Union[Unset, ColumnNullsPercentCheckSpec]):
        monthly_not_nulls_count (Union[Unset, ColumnNotNullsCountCheckSpec]):
        monthly_not_nulls_percent (Union[Unset, ColumnNotNullsPercentCheckSpec]):
    """

    monthly_nulls_count: Union[Unset, "ColumnNullsCountCheckSpec"] = UNSET
    monthly_nulls_percent: Union[Unset, "ColumnNullsPercentCheckSpec"] = UNSET
    monthly_not_nulls_count: Union[Unset, "ColumnNotNullsCountCheckSpec"] = UNSET
    monthly_not_nulls_percent: Union[Unset, "ColumnNotNullsPercentCheckSpec"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        monthly_nulls_count: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.monthly_nulls_count, Unset):
            monthly_nulls_count = self.monthly_nulls_count.to_dict()

        monthly_nulls_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.monthly_nulls_percent, Unset):
            monthly_nulls_percent = self.monthly_nulls_percent.to_dict()

        monthly_not_nulls_count: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.monthly_not_nulls_count, Unset):
            monthly_not_nulls_count = self.monthly_not_nulls_count.to_dict()

        monthly_not_nulls_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.monthly_not_nulls_percent, Unset):
            monthly_not_nulls_percent = self.monthly_not_nulls_percent.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if monthly_nulls_count is not UNSET:
            field_dict["monthly_nulls_count"] = monthly_nulls_count
        if monthly_nulls_percent is not UNSET:
            field_dict["monthly_nulls_percent"] = monthly_nulls_percent
        if monthly_not_nulls_count is not UNSET:
            field_dict["monthly_not_nulls_count"] = monthly_not_nulls_count
        if monthly_not_nulls_percent is not UNSET:
            field_dict["monthly_not_nulls_percent"] = monthly_not_nulls_percent

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_not_nulls_count_check_spec import (
            ColumnNotNullsCountCheckSpec,
        )
        from ..models.column_not_nulls_percent_check_spec import (
            ColumnNotNullsPercentCheckSpec,
        )
        from ..models.column_nulls_count_check_spec import ColumnNullsCountCheckSpec
        from ..models.column_nulls_percent_check_spec import ColumnNullsPercentCheckSpec

        d = src_dict.copy()
        _monthly_nulls_count = d.pop("monthly_nulls_count", UNSET)
        monthly_nulls_count: Union[Unset, ColumnNullsCountCheckSpec]
        if isinstance(_monthly_nulls_count, Unset):
            monthly_nulls_count = UNSET
        else:
            monthly_nulls_count = ColumnNullsCountCheckSpec.from_dict(
                _monthly_nulls_count
            )

        _monthly_nulls_percent = d.pop("monthly_nulls_percent", UNSET)
        monthly_nulls_percent: Union[Unset, ColumnNullsPercentCheckSpec]
        if isinstance(_monthly_nulls_percent, Unset):
            monthly_nulls_percent = UNSET
        else:
            monthly_nulls_percent = ColumnNullsPercentCheckSpec.from_dict(
                _monthly_nulls_percent
            )

        _monthly_not_nulls_count = d.pop("monthly_not_nulls_count", UNSET)
        monthly_not_nulls_count: Union[Unset, ColumnNotNullsCountCheckSpec]
        if isinstance(_monthly_not_nulls_count, Unset):
            monthly_not_nulls_count = UNSET
        else:
            monthly_not_nulls_count = ColumnNotNullsCountCheckSpec.from_dict(
                _monthly_not_nulls_count
            )

        _monthly_not_nulls_percent = d.pop("monthly_not_nulls_percent", UNSET)
        monthly_not_nulls_percent: Union[Unset, ColumnNotNullsPercentCheckSpec]
        if isinstance(_monthly_not_nulls_percent, Unset):
            monthly_not_nulls_percent = UNSET
        else:
            monthly_not_nulls_percent = ColumnNotNullsPercentCheckSpec.from_dict(
                _monthly_not_nulls_percent
            )

        column_nulls_monthly_recurring_checks_spec = cls(
            monthly_nulls_count=monthly_nulls_count,
            monthly_nulls_percent=monthly_nulls_percent,
            monthly_not_nulls_count=monthly_not_nulls_count,
            monthly_not_nulls_percent=monthly_not_nulls_percent,
        )

        column_nulls_monthly_recurring_checks_spec.additional_properties = d
        return column_nulls_monthly_recurring_checks_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.column_accuracy_monthly_recurring_checks_spec import (
        ColumnAccuracyMonthlyRecurringChecksSpec,
    )
    from ..models.column_anomaly_monthly_recurring_checks_spec import (
        ColumnAnomalyMonthlyRecurringChecksSpec,
    )
    from ..models.column_bool_monthly_recurring_checks_spec import (
        ColumnBoolMonthlyRecurringChecksSpec,
    )
    from ..models.column_datatype_monthly_recurring_checks_spec import (
        ColumnDatatypeMonthlyRecurringChecksSpec,
    )
    from ..models.column_datetime_monthly_recurring_checks_spec import (
        ColumnDatetimeMonthlyRecurringChecksSpec,
    )
    from ..models.column_integrity_monthly_recurring_checks_spec import (
        ColumnIntegrityMonthlyRecurringChecksSpec,
    )
    from ..models.column_monthly_recurring_check_categories_spec_comparisons import (
        ColumnMonthlyRecurringCheckCategoriesSpecComparisons,
    )
    from ..models.column_monthly_recurring_check_categories_spec_custom import (
        ColumnMonthlyRecurringCheckCategoriesSpecCustom,
    )
    from ..models.column_nulls_monthly_recurring_checks_spec import (
        ColumnNullsMonthlyRecurringChecksSpec,
    )
    from ..models.column_numeric_monthly_recurring_checks_spec import (
        ColumnNumericMonthlyRecurringChecksSpec,
    )
    from ..models.column_pii_monthly_recurring_checks_spec import (
        ColumnPiiMonthlyRecurringChecksSpec,
    )
    from ..models.column_schema_monthly_recurring_checks_spec import (
        ColumnSchemaMonthlyRecurringChecksSpec,
    )
    from ..models.column_sql_monthly_recurring_checks_spec import (
        ColumnSqlMonthlyRecurringChecksSpec,
    )
    from ..models.column_strings_monthly_recurring_checks_spec import (
        ColumnStringsMonthlyRecurringChecksSpec,
    )
    from ..models.column_uniqueness_monthly_recurring_checks_spec import (
        ColumnUniquenessMonthlyRecurringChecksSpec,
    )


T = TypeVar("T", bound="ColumnMonthlyRecurringCheckCategoriesSpec")


@attr.s(auto_attribs=True)
class ColumnMonthlyRecurringCheckCategoriesSpec:
    """
    Attributes:
        custom (Union[Unset, ColumnMonthlyRecurringCheckCategoriesSpecCustom]): Dictionary of custom checks. The keys
            are check names.
        nulls (Union[Unset, ColumnNullsMonthlyRecurringChecksSpec]):
        numeric (Union[Unset, ColumnNumericMonthlyRecurringChecksSpec]):
        strings (Union[Unset, ColumnStringsMonthlyRecurringChecksSpec]):
        uniqueness (Union[Unset, ColumnUniquenessMonthlyRecurringChecksSpec]):
        datetime_ (Union[Unset, ColumnDatetimeMonthlyRecurringChecksSpec]):
        pii (Union[Unset, ColumnPiiMonthlyRecurringChecksSpec]):
        sql (Union[Unset, ColumnSqlMonthlyRecurringChecksSpec]):
        bool_ (Union[Unset, ColumnBoolMonthlyRecurringChecksSpec]):
        integrity (Union[Unset, ColumnIntegrityMonthlyRecurringChecksSpec]):
        accuracy (Union[Unset, ColumnAccuracyMonthlyRecurringChecksSpec]):
        datatype (Union[Unset, ColumnDatatypeMonthlyRecurringChecksSpec]):
        anomaly (Union[Unset, ColumnAnomalyMonthlyRecurringChecksSpec]):
        schema (Union[Unset, ColumnSchemaMonthlyRecurringChecksSpec]):
        comparisons (Union[Unset, ColumnMonthlyRecurringCheckCategoriesSpecComparisons]): Dictionary of configuration of
            checks for table comparisons at a column level. The key that identifies each comparison must match the name of a
            data comparison that is configured on the parent table.
    """

    custom: Union[Unset, "ColumnMonthlyRecurringCheckCategoriesSpecCustom"] = UNSET
    nulls: Union[Unset, "ColumnNullsMonthlyRecurringChecksSpec"] = UNSET
    numeric: Union[Unset, "ColumnNumericMonthlyRecurringChecksSpec"] = UNSET
    strings: Union[Unset, "ColumnStringsMonthlyRecurringChecksSpec"] = UNSET
    uniqueness: Union[Unset, "ColumnUniquenessMonthlyRecurringChecksSpec"] = UNSET
    datetime_: Union[Unset, "ColumnDatetimeMonthlyRecurringChecksSpec"] = UNSET
    pii: Union[Unset, "ColumnPiiMonthlyRecurringChecksSpec"] = UNSET
    sql: Union[Unset, "ColumnSqlMonthlyRecurringChecksSpec"] = UNSET
    bool_: Union[Unset, "ColumnBoolMonthlyRecurringChecksSpec"] = UNSET
    integrity: Union[Unset, "ColumnIntegrityMonthlyRecurringChecksSpec"] = UNSET
    accuracy: Union[Unset, "ColumnAccuracyMonthlyRecurringChecksSpec"] = UNSET
    datatype: Union[Unset, "ColumnDatatypeMonthlyRecurringChecksSpec"] = UNSET
    anomaly: Union[Unset, "ColumnAnomalyMonthlyRecurringChecksSpec"] = UNSET
    schema: Union[Unset, "ColumnSchemaMonthlyRecurringChecksSpec"] = UNSET
    comparisons: Union[
        Unset, "ColumnMonthlyRecurringCheckCategoriesSpecComparisons"
    ] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        custom: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.custom, Unset):
            custom = self.custom.to_dict()

        nulls: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.nulls, Unset):
            nulls = self.nulls.to_dict()

        numeric: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.numeric, Unset):
            numeric = self.numeric.to_dict()

        strings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.strings, Unset):
            strings = self.strings.to_dict()

        uniqueness: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.uniqueness, Unset):
            uniqueness = self.uniqueness.to_dict()

        datetime_: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.datetime_, Unset):
            datetime_ = self.datetime_.to_dict()

        pii: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pii, Unset):
            pii = self.pii.to_dict()

        sql: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.sql, Unset):
            sql = self.sql.to_dict()

        bool_: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.bool_, Unset):
            bool_ = self.bool_.to_dict()

        integrity: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.integrity, Unset):
            integrity = self.integrity.to_dict()

        accuracy: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.accuracy, Unset):
            accuracy = self.accuracy.to_dict()

        datatype: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.datatype, Unset):
            datatype = self.datatype.to_dict()

        anomaly: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.anomaly, Unset):
            anomaly = self.anomaly.to_dict()

        schema: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.schema, Unset):
            schema = self.schema.to_dict()

        comparisons: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.comparisons, Unset):
            comparisons = self.comparisons.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if custom is not UNSET:
            field_dict["custom"] = custom
        if nulls is not UNSET:
            field_dict["nulls"] = nulls
        if numeric is not UNSET:
            field_dict["numeric"] = numeric
        if strings is not UNSET:
            field_dict["strings"] = strings
        if uniqueness is not UNSET:
            field_dict["uniqueness"] = uniqueness
        if datetime_ is not UNSET:
            field_dict["datetime"] = datetime_
        if pii is not UNSET:
            field_dict["pii"] = pii
        if sql is not UNSET:
            field_dict["sql"] = sql
        if bool_ is not UNSET:
            field_dict["bool"] = bool_
        if integrity is not UNSET:
            field_dict["integrity"] = integrity
        if accuracy is not UNSET:
            field_dict["accuracy"] = accuracy
        if datatype is not UNSET:
            field_dict["datatype"] = datatype
        if anomaly is not UNSET:
            field_dict["anomaly"] = anomaly
        if schema is not UNSET:
            field_dict["schema"] = schema
        if comparisons is not UNSET:
            field_dict["comparisons"] = comparisons

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_accuracy_monthly_recurring_checks_spec import (
            ColumnAccuracyMonthlyRecurringChecksSpec,
        )
        from ..models.column_anomaly_monthly_recurring_checks_spec import (
            ColumnAnomalyMonthlyRecurringChecksSpec,
        )
        from ..models.column_bool_monthly_recurring_checks_spec import (
            ColumnBoolMonthlyRecurringChecksSpec,
        )
        from ..models.column_datatype_monthly_recurring_checks_spec import (
            ColumnDatatypeMonthlyRecurringChecksSpec,
        )
        from ..models.column_datetime_monthly_recurring_checks_spec import (
            ColumnDatetimeMonthlyRecurringChecksSpec,
        )
        from ..models.column_integrity_monthly_recurring_checks_spec import (
            ColumnIntegrityMonthlyRecurringChecksSpec,
        )
        from ..models.column_monthly_recurring_check_categories_spec_comparisons import (
            ColumnMonthlyRecurringCheckCategoriesSpecComparisons,
        )
        from ..models.column_monthly_recurring_check_categories_spec_custom import (
            ColumnMonthlyRecurringCheckCategoriesSpecCustom,
        )
        from ..models.column_nulls_monthly_recurring_checks_spec import (
            ColumnNullsMonthlyRecurringChecksSpec,
        )
        from ..models.column_numeric_monthly_recurring_checks_spec import (
            ColumnNumericMonthlyRecurringChecksSpec,
        )
        from ..models.column_pii_monthly_recurring_checks_spec import (
            ColumnPiiMonthlyRecurringChecksSpec,
        )
        from ..models.column_schema_monthly_recurring_checks_spec import (
            ColumnSchemaMonthlyRecurringChecksSpec,
        )
        from ..models.column_sql_monthly_recurring_checks_spec import (
            ColumnSqlMonthlyRecurringChecksSpec,
        )
        from ..models.column_strings_monthly_recurring_checks_spec import (
            ColumnStringsMonthlyRecurringChecksSpec,
        )
        from ..models.column_uniqueness_monthly_recurring_checks_spec import (
            ColumnUniquenessMonthlyRecurringChecksSpec,
        )

        d = src_dict.copy()
        _custom = d.pop("custom", UNSET)
        custom: Union[Unset, ColumnMonthlyRecurringCheckCategoriesSpecCustom]
        if isinstance(_custom, Unset):
            custom = UNSET
        else:
            custom = ColumnMonthlyRecurringCheckCategoriesSpecCustom.from_dict(_custom)

        _nulls = d.pop("nulls", UNSET)
        nulls: Union[Unset, ColumnNullsMonthlyRecurringChecksSpec]
        if isinstance(_nulls, Unset):
            nulls = UNSET
        else:
            nulls = ColumnNullsMonthlyRecurringChecksSpec.from_dict(_nulls)

        _numeric = d.pop("numeric", UNSET)
        numeric: Union[Unset, ColumnNumericMonthlyRecurringChecksSpec]
        if isinstance(_numeric, Unset):
            numeric = UNSET
        else:
            numeric = ColumnNumericMonthlyRecurringChecksSpec.from_dict(_numeric)

        _strings = d.pop("strings", UNSET)
        strings: Union[Unset, ColumnStringsMonthlyRecurringChecksSpec]
        if isinstance(_strings, Unset):
            strings = UNSET
        else:
            strings = ColumnStringsMonthlyRecurringChecksSpec.from_dict(_strings)

        _uniqueness = d.pop("uniqueness", UNSET)
        uniqueness: Union[Unset, ColumnUniquenessMonthlyRecurringChecksSpec]
        if isinstance(_uniqueness, Unset):
            uniqueness = UNSET
        else:
            uniqueness = ColumnUniquenessMonthlyRecurringChecksSpec.from_dict(
                _uniqueness
            )

        _datetime_ = d.pop("datetime", UNSET)
        datetime_: Union[Unset, ColumnDatetimeMonthlyRecurringChecksSpec]
        if isinstance(_datetime_, Unset):
            datetime_ = UNSET
        else:
            datetime_ = ColumnDatetimeMonthlyRecurringChecksSpec.from_dict(_datetime_)

        _pii = d.pop("pii", UNSET)
        pii: Union[Unset, ColumnPiiMonthlyRecurringChecksSpec]
        if isinstance(_pii, Unset):
            pii = UNSET
        else:
            pii = ColumnPiiMonthlyRecurringChecksSpec.from_dict(_pii)

        _sql = d.pop("sql", UNSET)
        sql: Union[Unset, ColumnSqlMonthlyRecurringChecksSpec]
        if isinstance(_sql, Unset):
            sql = UNSET
        else:
            sql = ColumnSqlMonthlyRecurringChecksSpec.from_dict(_sql)

        _bool_ = d.pop("bool", UNSET)
        bool_: Union[Unset, ColumnBoolMonthlyRecurringChecksSpec]
        if isinstance(_bool_, Unset):
            bool_ = UNSET
        else:
            bool_ = ColumnBoolMonthlyRecurringChecksSpec.from_dict(_bool_)

        _integrity = d.pop("integrity", UNSET)
        integrity: Union[Unset, ColumnIntegrityMonthlyRecurringChecksSpec]
        if isinstance(_integrity, Unset):
            integrity = UNSET
        else:
            integrity = ColumnIntegrityMonthlyRecurringChecksSpec.from_dict(_integrity)

        _accuracy = d.pop("accuracy", UNSET)
        accuracy: Union[Unset, ColumnAccuracyMonthlyRecurringChecksSpec]
        if isinstance(_accuracy, Unset):
            accuracy = UNSET
        else:
            accuracy = ColumnAccuracyMonthlyRecurringChecksSpec.from_dict(_accuracy)

        _datatype = d.pop("datatype", UNSET)
        datatype: Union[Unset, ColumnDatatypeMonthlyRecurringChecksSpec]
        if isinstance(_datatype, Unset):
            datatype = UNSET
        else:
            datatype = ColumnDatatypeMonthlyRecurringChecksSpec.from_dict(_datatype)

        _anomaly = d.pop("anomaly", UNSET)
        anomaly: Union[Unset, ColumnAnomalyMonthlyRecurringChecksSpec]
        if isinstance(_anomaly, Unset):
            anomaly = UNSET
        else:
            anomaly = ColumnAnomalyMonthlyRecurringChecksSpec.from_dict(_anomaly)

        _schema = d.pop("schema", UNSET)
        schema: Union[Unset, ColumnSchemaMonthlyRecurringChecksSpec]
        if isinstance(_schema, Unset):
            schema = UNSET
        else:
            schema = ColumnSchemaMonthlyRecurringChecksSpec.from_dict(_schema)

        _comparisons = d.pop("comparisons", UNSET)
        comparisons: Union[Unset, ColumnMonthlyRecurringCheckCategoriesSpecComparisons]
        if isinstance(_comparisons, Unset):
            comparisons = UNSET
        else:
            comparisons = (
                ColumnMonthlyRecurringCheckCategoriesSpecComparisons.from_dict(
                    _comparisons
                )
            )

        column_monthly_recurring_check_categories_spec = cls(
            custom=custom,
            nulls=nulls,
            numeric=numeric,
            strings=strings,
            uniqueness=uniqueness,
            datetime_=datetime_,
            pii=pii,
            sql=sql,
            bool_=bool_,
            integrity=integrity,
            accuracy=accuracy,
            datatype=datatype,
            anomaly=anomaly,
            schema=schema,
            comparisons=comparisons,
        )

        column_monthly_recurring_check_categories_spec.additional_properties = d
        return column_monthly_recurring_check_categories_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.table_accuracy_total_row_count_match_percent_check_spec import (
        TableAccuracyTotalRowCountMatchPercentCheckSpec,
    )


T = TypeVar("T", bound="TableAccuracyDailyRecurringChecksSpec")


@attr.s(auto_attribs=True)
class TableAccuracyDailyRecurringChecksSpec:
    """
    Attributes:
        daily_total_row_count_match_percent (Union[Unset, TableAccuracyTotalRowCountMatchPercentCheckSpec]):
    """

    daily_total_row_count_match_percent: Union[
        Unset, "TableAccuracyTotalRowCountMatchPercentCheckSpec"
    ] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        daily_total_row_count_match_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_total_row_count_match_percent, Unset):
            daily_total_row_count_match_percent = (
                self.daily_total_row_count_match_percent.to_dict()
            )

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if daily_total_row_count_match_percent is not UNSET:
            field_dict[
                "daily_total_row_count_match_percent"
            ] = daily_total_row_count_match_percent

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.table_accuracy_total_row_count_match_percent_check_spec import (
            TableAccuracyTotalRowCountMatchPercentCheckSpec,
        )

        d = src_dict.copy()
        _daily_total_row_count_match_percent = d.pop(
            "daily_total_row_count_match_percent", UNSET
        )
        daily_total_row_count_match_percent: Union[
            Unset, TableAccuracyTotalRowCountMatchPercentCheckSpec
        ]
        if isinstance(_daily_total_row_count_match_percent, Unset):
            daily_total_row_count_match_percent = UNSET
        else:
            daily_total_row_count_match_percent = (
                TableAccuracyTotalRowCountMatchPercentCheckSpec.from_dict(
                    _daily_total_row_count_match_percent
                )
            )

        table_accuracy_daily_recurring_checks_spec = cls(
            daily_total_row_count_match_percent=daily_total_row_count_match_percent,
        )

        table_accuracy_daily_recurring_checks_spec.additional_properties = d
        return table_accuracy_daily_recurring_checks_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="DqoUserProfileModel")


@attr.s(auto_attribs=True)
class DqoUserProfileModel:
    """The model that describes the current user and his access rights.

    Attributes:
        user (Union[Unset, str]): User email.
        tenant (Union[Unset, str]): DQO Cloud tenant.
        license_type (Union[Unset, str]): DQO Cloud license type.
        trial_period_expires_at (Union[Unset, str]): The date and time when the trial period of a PERSONAL DQO license
            expires and the account is downgraded to a FREE license.
        connections_limit (Union[Unset, int]): Limit of the number of connections that could be synchronized to the DQO
            Cloud data quality warehouse.
        users_limit (Union[Unset, int]): Limit of the number of users that could be added to a DQO environment.
        months_limit (Union[Unset, int]): Limit of the number of recent months (excluding the current month) that could
            be synchronized to the DQO Cloud data quality warehouse.
        connection_tables_limit (Union[Unset, int]): Limit of the number of tables inside each connection that could be
            synchronized to the DQO Cloud data quality warehouse.
        tables_limit (Union[Unset, int]): Limit of the total number of tables that could be synchronized to the DQO
            Cloud data quality warehouse.
        jobs_limit (Union[Unset, int]): Limit of the number of supported concurrent jobs that DQO can run in parallel on
            this instance.
    """

    user: Union[Unset, str] = UNSET
    tenant: Union[Unset, str] = UNSET
    license_type: Union[Unset, str] = UNSET
    trial_period_expires_at: Union[Unset, str] = UNSET
    connections_limit: Union[Unset, int] = UNSET
    users_limit: Union[Unset, int] = UNSET
    months_limit: Union[Unset, int] = UNSET
    connection_tables_limit: Union[Unset, int] = UNSET
    tables_limit: Union[Unset, int] = UNSET
    jobs_limit: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user = self.user
        tenant = self.tenant
        license_type = self.license_type
        trial_period_expires_at = self.trial_period_expires_at
        connections_limit = self.connections_limit
        users_limit = self.users_limit
        months_limit = self.months_limit
        connection_tables_limit = self.connection_tables_limit
        tables_limit = self.tables_limit
        jobs_limit = self.jobs_limit

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if user is not UNSET:
            field_dict["user"] = user
        if tenant is not UNSET:
            field_dict["tenant"] = tenant
        if license_type is not UNSET:
            field_dict["license_type"] = license_type
        if trial_period_expires_at is not UNSET:
            field_dict["trial_period_expires_at"] = trial_period_expires_at
        if connections_limit is not UNSET:
            field_dict["connections_limit"] = connections_limit
        if users_limit is not UNSET:
            field_dict["users_limit"] = users_limit
        if months_limit is not UNSET:
            field_dict["months_limit"] = months_limit
        if connection_tables_limit is not UNSET:
            field_dict["connection_tables_limit"] = connection_tables_limit
        if tables_limit is not UNSET:
            field_dict["tables_limit"] = tables_limit
        if jobs_limit is not UNSET:
            field_dict["jobs_limit"] = jobs_limit

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        user = d.pop("user", UNSET)

        tenant = d.pop("tenant", UNSET)

        license_type = d.pop("license_type", UNSET)

        trial_period_expires_at = d.pop("trial_period_expires_at", UNSET)

        connections_limit = d.pop("connections_limit", UNSET)

        users_limit = d.pop("users_limit", UNSET)

        months_limit = d.pop("months_limit", UNSET)

        connection_tables_limit = d.pop("connection_tables_limit", UNSET)

        tables_limit = d.pop("tables_limit", UNSET)

        jobs_limit = d.pop("jobs_limit", UNSET)

        dqo_user_profile_model = cls(
            user=user,
            tenant=tenant,
            license_type=license_type,
            trial_period_expires_at=trial_period_expires_at,
            connections_limit=connections_limit,
            users_limit=users_limit,
            months_limit=months_limit,
            connection_tables_limit=connection_tables_limit,
            tables_limit=tables_limit,
            jobs_limit=jobs_limit,
        )

        dqo_user_profile_model.additional_properties = d
        return dqo_user_profile_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
